package org.processmining.starstar.utilities;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.starstar.elements.XOCEvent;
import org.processmining.starstar.elements.XOCObject;
import org.processmining.starstar.elements.XOCObjectClass;
import org.processmining.starstar.logExtraction.StarStarTrace;

public class LogExtractionFromModel {
	String chosenClass;
	Map<XOCObject, List<XOCObject>> intersectedObjects;
	Map<String, XOCObjectClass> objectClasses;
	List<StarStarTrace> traces;
	
	public LogExtractionFromModel(String chosenClass, Map<XOCObject, List<XOCObject>> intersectedObjects, Map<String, XOCObjectClass> objectClasses) {
		this.chosenClass = chosenClass;
		this.intersectedObjects = intersectedObjects;
		this.objectClasses = objectClasses;
		this.traces = new ArrayList<StarStarTrace>();
		this.formInitialTraces();
	}
	
	public String getChosenClass() {
		return this.chosenClass;
	}
	
	public Map<XOCObject, List<XOCObject>> getIntersectedObjects() {
		return this.intersectedObjects;
	}
	
	public Map<String, XOCObjectClass> getObjectClasses() {
		return this.objectClasses;
	}
	
	public List<StarStarTrace> getTraces() {
		return this.traces;
	}
	
	public void formInitialTraces() {
		for (XOCObject object : objectClasses.get(chosenClass).getSubscribedObjects()) {
			traces.add(new StarStarTrace(this.intersectedObjects, object));
		}
	}
	
	public void applyIterationOnTraces(Double w, int nit) {
		if (nit > 0) {
			for (StarStarTrace str : this.traces) {
				List<XOCObject> objectsToAdd = new ArrayList<XOCObject>();
				for (XOCObject obj1 : str.getInvolvedObjects()) {
					List<XOCObject> intersectedObjects = this.intersectedObjects.get(obj1);
					for (XOCObject obj2 : intersectedObjects) {
						if (obj2.getWeight() > w) {
							if (!str.getInvolvedObjects().contains(obj2) && !objectsToAdd.contains(obj2)) {
								objectsToAdd.add(obj2);
							}
						}
					}
				}
				int oldTraceSize = str.getInvolvedObjects().size();
				
				for (XOCObject object : objectsToAdd) {
					str.addInvolvedObject(object);
				}
				
				int newTraceSize = str.getInvolvedObjects().size();
			}
			this.cleanTracesByObjects();
			this.cleanTracesByEvents();
			if (nit > 1) {
				applyIterationOnTraces(w, nit-1);
			}
		}
		this.cleanTracesByEvents();
	}
	
	public void cleanTracesByObjects() {
		List<String> alreadyEvaluatedTraces = new ArrayList<String>();
		int i = 0;
		while (i < this.traces.size()) {
			StarStarTrace trace = this.traces.get(i);
			String traceRepresentation = trace.getStarStarObjectsRepresentation();
			if (!alreadyEvaluatedTraces.contains(traceRepresentation)) {
				alreadyEvaluatedTraces.add(traceRepresentation);
			}
			else {
				this.traces.remove(i);
				continue;
			}
			i++;
		}
	}
	
	public void cleanTracesByEvents() {
		List<String> alreadyEvaluatedTraces = new ArrayList<String>();
		int i = 0;
		while (i < this.traces.size()) {
			StarStarTrace trace = this.traces.get(i);
			trace.formInvolvedEvents();
			String traceRepresentation = trace.getStarStarEventsRepresentation();
			if (!alreadyEvaluatedTraces.contains(traceRepresentation)) {
				alreadyEvaluatedTraces.add(traceRepresentation);
			}
			else {
				this.traces.remove(i);
				continue;
			}
			i++;
		}
	}
	
	public void printTraceRepresentations(String reason) {
		for (StarStarTrace trace : this.traces) {
			trace.printTraceRepresentation(reason);
		}
	}
	
	public XLog getEventualLog() {
		XAttributeMapImpl attributeMap = new XAttributeMapImpl();
		XLog log = new XLogImpl(attributeMap);
		log.getAttributes().put("concept:name", new XAttributeLiteralImpl("concept:name", "testlog"));
		
		int traceCount = 0;
		
		//this.printTraceRepresentations("formingLog");
		
		for (StarStarTrace str : this.traces) {
			XAttributeMapImpl traceAttributeMap = new XAttributeMapImpl();
			str.formInvolvedEvents();
			
			XTrace tr = new XTraceImpl(traceAttributeMap);
			tr.getAttributes().put("concept:name", new XAttributeLiteralImpl("concept:name", ""+traceCount));
			str.sortEventsByTimestamp();
			
			for (XOCEvent eve : str.getInvolvedEvents()) {
				XAttributeMapImpl eventAttributeMap = new XAttributeMapImpl();
				XEvent ev = new XEventImpl(eventAttributeMap);
				System.out.println(eve.getActivity().getActivityLabel());
				
				ev.getAttributes().put("concept:name", new XAttributeLiteralImpl("concept:name", eve.getActivity().getActivityLabel()));
				ev.getAttributes().put("time:timestamp", new XAttributeTimestampImpl("time:timestamp", eve.getTimestamp()));
				
				tr.add(ev);
			}
			
			log.add(tr);

			traceCount++;
		}
		
		return log;
	}
}
