package org.processmining.starstar.visualizer;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.extension.std.XExtendedEvent;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.pnalignanalysis.visualization.projection.util.GraphBuilder;
import org.processmining.starstar.constants.ModelConstants;
import org.processmining.starstar.elements.XOCActivity;
import org.processmining.starstar.model.ModelEdge;
import org.processmining.starstar.model.StarStarModel;
import org.processmining.starstar.utilities.GetLogFromLog;
import org.processmining.starstar.utilities.GetNetFromLogUsingIM;
import org.processmining.starstar.utilities.LogExtractionFromModel;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.view.mxGraph;

@Plugin(name = "Star Star Model Visualizer", parameterLabels = { "StarStarModel" }, returnLabels = { "JPanel" }, returnTypes = { JPanel.class })
@Visualizer
public class StarStarModelVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JPanel visualize(PluginContext context, StarStarModel model) {
		MainViewStarStar view = new MainViewStarStar(context, model);
		
		return view;
	}
}

class MainViewStarStar extends JPanel {
	PluginContext context;
	StarStarModel model;
	
	LeftPanelStarStar leftPanel;
	RightPanelStarStar rightPanel;
	
	public MainViewStarStar(PluginContext context, StarStarModel model) {
		this.context = context;
		this.model = model;
		
		RelativeLayout rl = new RelativeLayout(RelativeLayout.X_AXIS);
		rl.setFill( true );
		this.setLayout(rl);
		
		this.leftPanel = new LeftPanelStarStar(context, model);
		this.rightPanel = new RightPanelStarStar(context, model);
		
		this.add(this.leftPanel, new Float(70));
		this.add(this.rightPanel, new Float(30));
		
		this.leftPanel.setAware(this.rightPanel);
	}
}

class LeftPanelStarStar extends JTabbedPane {
	PluginContext context;
	StarStarModel model;
	
	RightPanelStarStar rightPanel;
	ModelVisualizerStarStar visualizer;
	
	Map<String, JPanel> addedJPanels;
	
	public LeftPanelStarStar(PluginContext context, StarStarModel model) {
		this.context = context;
		this.model = model;
		
		/*RelativeLayout rl = new RelativeLayout(RelativeLayout.X_AXIS);
		rl.setFill( true );
		this.setLayout(rl);*/
		
		this.addedJPanels = new HashMap<String, JPanel>();
		
		this.visualizer = new ModelVisualizerStarStar(context, model);
		
		this.addJPanel("** Model", this.visualizer);
		
		//this.add(this.visualizer, new Float(100));
	}
	
	public void setAware(RightPanelStarStar rightPanel) {
		this.rightPanel = rightPanel;
		this.visualizer.setAware(this.rightPanel);
		this.rightPanel.setAware(this);
	}
	
	public Map<String, JPanel> getAddedJPanels() {
		return this.addedJPanels;
	}
	
	public void addJPanel(String label, JPanel panel) {
		this.addedJPanels.put(label, panel);
		this.add(label, panel);
	}
}

class ModelVisualizerStarStar extends JPanel {
	PluginContext context;
	StarStarModel model;
	mxGraph graph;
	mxGraphComponent graphComponent;
	Map<XOCActivity, Object> nodesInTheGraph;
	Map<Object, XOCActivity> invNodesInTheGraph;
	Map<ModelEdge, Object> edgesInTheGraph;
	Map<Object, ModelEdge> invEdgesInTheGraph;
	JScrollPane scrollPane;
	String perspective;
	Double w;
	Integer minCount;
	Integer parallelEdgeSpacing;
	Integer interRankCellSpacing;
	Integer maxCountPerEdge;
	Integer interHierarchySpacing;
	Integer intraCellSpacing;
	boolean showCountOnActivity;
	Integer minActivityCount;
	Integer maxActivityCount;
	
	RightPanelStarStar rightPanel;
	GraphMouseListener graphMouseListener;
	
	public ModelVisualizerStarStar(PluginContext context, StarStarModel model) {
		this.context = context;
		this.model = model;
		
		this.nodesInTheGraph = new HashMap<XOCActivity, Object>();
		this.invNodesInTheGraph = new HashMap<Object, XOCActivity>();
		this.edgesInTheGraph = new HashMap<ModelEdge, Object>();
		this.invEdgesInTheGraph = new HashMap<Object, ModelEdge>();
		
		this.setLayout(new GridLayout());
		
		perspective = ModelConstants.defaultPerspective;
		w = ModelConstants.defaultW;
		minCount = ModelConstants.defaultMinCount;
		parallelEdgeSpacing = ModelConstants.defaultParallelEdgeSpacing;
		interRankCellSpacing = ModelConstants.defaultInterRankCellSpacing;
		maxCountPerEdge = ModelConstants.defaultMinCount;
		interHierarchySpacing = ModelConstants.defaultInterHierarchySpacing;
		intraCellSpacing = ModelConstants.defaultIntraCellSpacing;
		showCountOnActivity = ModelConstants.defaultShowCountOnActivities;
		minActivityCount = ModelConstants.defaultMinActivityCount;
		maxActivityCount = ModelConstants.defaultMinActivityCount;
	}
	
	public void setAware(RightPanelStarStar rightPanel) {
		this.rightPanel = rightPanel;
		
		doRepresentationWork();
		addGraphToView();
	}
	
	public void setPerspective(String perspective) {
		this.perspective = perspective;
	}
	
	public void setW(Double w) {
		this.w = w;
	}
	
	public void setMinCount(Integer minCount) {
		this.minCount = minCount;
	}
	
	public void setMinActivityCount(Integer minActivityCount) {
		this.minActivityCount = minActivityCount;
	}
	
	public void setParallelEdgeSpacing(Integer parallelEdgeSpacing) {
		this.parallelEdgeSpacing = parallelEdgeSpacing;
	}
	
	public void setInterRankCellSpacing(Integer interRankCellSpacing) {
		this.interRankCellSpacing = interRankCellSpacing;
	}
	
	public void setInterHierarchySpacing(Integer interHierarchySpacing) {
		this.interHierarchySpacing = interHierarchySpacing;
	}
	
	public void setIntraCellSpacing(Integer intraCellSpacing) {
		this.intraCellSpacing = intraCellSpacing;
	}
	
	public void setShowCountOnActivities(boolean value) {
		this.showCountOnActivity = value;
	}
	
	public void doRepresentationWork() {
		beginGraph();
		drawGraph();
		doLayouting();
	}
	
	public void beginGraph() {
		if (this.graphComponent != null) {
			this.scrollPane.remove(this.graphComponent);
			//this.scrollPane.updateUI();
			this.remove(this.scrollPane);
			//this.updateUI();
		}
		graph = new mxGraph();
		graph.getModel().beginUpdate();
	}
	
	public String getHexChar(int term) {
		if (term < 10) {
			return ""+term;
		}
		else if (term == 10) {
			return "A";
		}
		else if (term == 11) {
			return "B";
		}
		else if (term == 12) {
			return "C";
		}
		else if (term == 13) {
			return "D";
		}
		else if (term == 14) {
			return "E";
		}
		return "F";
	}
	
	public String getHexString(int inte) {		
		int firstTerm = inte / 16;
		int secondTerm = inte % 16;
		
		String hexString = getHexChar(firstTerm) + getHexChar(secondTerm);
		
		return hexString;
	}
	
	public String getColourFromValueAndMinAndMax(int minValue, int maxValue, int currentValue, String defaultComponentValue, String mainComponent, double attenuationTerm) {
		String redPart = defaultComponentValue;
		String greenPart = defaultComponentValue;
		String bluePart = defaultComponentValue;
		
		if (maxValue > minValue && currentValue > minValue && minValue >= 1) {
			double logarithmicDiffMaxMin = Math.log(maxValue) - Math.log(minValue);
			double logarithmicDiffValueMin = Math.log(currentValue) - Math.log(minValue);
			
			double ratio0 = (255.0 * logarithmicDiffValueMin / logarithmicDiffMaxMin) * attenuationTerm;
			int ratio = (int) Math.floor(ratio0);
			int diffRatio = 255 - ratio;
			String ratioString = getHexString(ratio);
			String diffRatioString = getHexString(diffRatio);
			
			if (defaultComponentValue == "FF") {
				if (mainComponent == "blue") {
					bluePart = defaultComponentValue;
					greenPart = diffRatioString;
					redPart = diffRatioString;
				}
				else if (mainComponent == "green") {
					bluePart = diffRatioString;
					greenPart = defaultComponentValue;
					redPart = diffRatioString;
				}
				else if (mainComponent == "red") {
					bluePart = diffRatioString;
					greenPart = diffRatioString;
					redPart = defaultComponentValue;
				}
			}
			else {
				if (mainComponent == "blue") {
					bluePart = ratioString;
					greenPart = defaultComponentValue;
					redPart = defaultComponentValue;
				}
				else if (mainComponent == "green") {
					bluePart = defaultComponentValue;
					greenPart = ratioString;
					redPart = defaultComponentValue;
				}
				else if (mainComponent == "red") {
					bluePart = defaultComponentValue;
					greenPart = defaultComponentValue;
					redPart = ratioString;
				}
			}
		}
		
		return "#" + redPart + greenPart + bluePart;
	}
	
	public int getStrokeWidthFromMinMaxAndValue(int minStrokeWidth, int maxStrokeWidth, double minValue, double maxValue, double currentValue) {
		int additionalWidth = 0;
		
		int diffMaxMinStrokeWidth = maxStrokeWidth - minStrokeWidth;
		
		if (currentValue == maxValue) {
			additionalWidth = diffMaxMinStrokeWidth;
		}
		else if (currentValue > minValue) {
			double logarithmicDiffMaxMin = Math.log(maxValue) - Math.log(minValue);
			double logarithmicDiffValueMin = Math.log(currentValue) - Math.log(minValue);
			
			additionalWidth = (int) (logarithmicDiffValueMin / logarithmicDiffValueMin * diffMaxMinStrokeWidth);
		}
		
		return minStrokeWidth + additionalWidth;
	}
	
	public void drawGraph() {
		Object parent = graph.getDefaultParent();
		
		List<XOCActivity> activitiesToAdd = new ArrayList<XOCActivity>();
		List<Integer> activitiesToAddCount = new ArrayList<Integer>();
		
		for (XOCActivity activity : model.getActivities()) {
			if (activity.getActivityCount() >= this.minActivityCount) {
				if (activity.getActivityCount() > this.maxActivityCount) {
					this.maxActivityCount = activity.getActivityCount();
				}
				activitiesToAdd.add(activity);
				activitiesToAddCount.add(activity.getActivityCount());
			}
		}
		
		if (activitiesToAdd.size() > 0) {
			Collections.sort(activitiesToAddCount);
			
			for (XOCActivity activity : activitiesToAdd) {
				String activityLabelOnObject = activity.getActivityLabel();
				if (showCountOnActivity) {
					activityLabelOnObject = activityLabelOnObject + " ("+activity.getActivityCount()+")";
				}
				//activityLabelOnObject = activityLabelOnObject.replaceAll("[^\\x00-\\x7F]", "");
				String activityColour = getColourFromValueAndMinAndMax(activitiesToAddCount.get(0), activitiesToAddCount.get(activitiesToAddCount.size()-1), activity.getActivityCount(), "FF", "blue", 0.35);
				Object activityObject = graph.insertVertex(parent, activityLabelOnObject, activityLabelOnObject, 150, 150, 275, 60, "fillColor="+activityColour+";fontSize=18");
				//Object activityObject = graph.insertVertex(parent, activityLabelOnObject, activityLabelOnObject, 150, 150, 200, 42);
				this.nodesInTheGraph.put(activity, activityObject);
				this.invNodesInTheGraph.put(activityObject, activity);
			}
			
			List<ModelEdge> edgesToAdd = new ArrayList<ModelEdge>();
			List<Double> edgesDisplayValue = new ArrayList<Double>();
					
			for (XOCActivity activity : model.getActivities()) {
				for (ModelEdge edge : activity.getOutputModelEdges()) {
					if (edge.getCount() > this.maxCountPerEdge) {
						this.maxCountPerEdge = edge.getCount();
					}
					if (edge.getWeight() >= w && edge.getCount() >= minCount) {
						if (!edgesToAdd.contains(edge)) {
							if (this.nodesInTheGraph.containsKey(edge.getSourceActivity()) && this.nodesInTheGraph.containsKey(edge.getTargetActivity())) {
								edgesToAdd.add(edge);
								edgesDisplayValue.add(edge.getPerspectiveValue(perspective));
							}
						}
					}
				}
			}
			
			if (edgesToAdd.size() > 0) {
				Collections.sort(edgesDisplayValue);
				
				Integer insertedEdgesCount = 0;
				
				System.out.println(edgesToAdd.size());
				
				for (ModelEdge edge : edgesToAdd) {
					String arcColour = "";

					if (perspective.equals("performance")) {
						arcColour = getColourFromValueAndMinAndMax((int)(double)edgesDisplayValue.get(0), (int)(double)edgesDisplayValue.get(edgesDisplayValue.size()-1), (int)(double)edge.getPerspectiveValue(perspective), "00", "red", 0.85);						
					}
					else if (perspective.equals("weight")) {
						arcColour = getColourFromValueAndMinAndMax((int)(double)edgesDisplayValue.get(0), (int)(double)edgesDisplayValue.get(edgesDisplayValue.size()-1), (int)(double)edge.getPerspectiveValue(perspective), "00", "green", 0.85);
					}
					else {
						arcColour = getColourFromValueAndMinAndMax((int)(double)edgesDisplayValue.get(0), (int)(double)edgesDisplayValue.get(edgesDisplayValue.size()-1), (int)(double)edge.getPerspectiveValue(perspective), "00", "blue", 0.85);
					}
					
					int strokeWidth = getStrokeWidthFromMinMaxAndValue(1, 4, edgesDisplayValue.get(0), edgesDisplayValue.get(edgesDisplayValue.size()-1), edge.getPerspectiveValue(perspective));
					
					Object arc = graph.insertEdge(parent, null, edge.getEdgeLabel(perspective), this.nodesInTheGraph.get(edge.getSourceActivity()), this.nodesInTheGraph.get(edge.getTargetActivity()), "strokeColor="+arcColour+";strokeWidth="+strokeWidth+";fontSize=16");
					//System.out.println(this.nodesInTheGraph.get(edge.getSourceActivity()));
					//System.out.println(this.nodesInTheGraph.get(edge.getTargetActivity()));
					//System.out.println(edge.getEdgeLabel(perspective));
					//Object arc = graph.insertEdge(parent, null, "", this.nodesInTheGraph.get(edge.getSourceActivity()), this.nodesInTheGraph.get(edge.getTargetActivity()));

					insertedEdgesCount++;
					
					/*if (insertedEdgesCount > 33) {
						break;
					}*/
					
					this.edgesInTheGraph.put(edge, arc);
					this.invEdgesInTheGraph.put(arc, edge);
				}
			}
		}
	}
	
	public void doLayouting() {
		mxHierarchicalLayout layout = new mxHierarchicalLayout(graph);
		layout.setParallelEdgeSpacing(this.parallelEdgeSpacing);
		layout.setInterRankCellSpacing(this.interRankCellSpacing);
		layout.setInterHierarchySpacing(this.interHierarchySpacing);
		layout.setIntraCellSpacing(this.intraCellSpacing);

		layout.execute(graph.getDefaultParent());
		this.graph.getModel().endUpdate();
	}
	
	public void addGraphToView() {
		this.graphComponent = new mxGraphComponent(this.graph);
		
		this.scrollPane = new JScrollPane(this.graphComponent);
		this.scrollPane.setPreferredSize(new Dimension(800, 800));
		
		this.scrollPane.getViewport().setMinimumSize(new Dimension(160, 200));
		this.scrollPane.getViewport().setPreferredSize(new Dimension(160, 200));
		
		this.scrollPane.updateUI();
		this.add(this.scrollPane);
		this.updateUI();
		
		graphMouseListener = new GraphMouseListener(this.context, this.model, this, this.rightPanel);
		this.graphComponent.getGraphControl().addMouseListener(graphMouseListener);
	}
}

class GraphMouseListener implements MouseListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	
	public GraphMouseListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
	}
	 
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		//System.out.println("mouseReleased");
		Object cell = this.visualizer.graphComponent.getCellAt(e.getX(), e.getY());
		this.rightPanel.destroyLogExtractionPerspective();
		this.rightPanel.destroyActivityFilterPanel();
		this.rightPanel.destroySubgraphVisualizer();
		
		if (this.visualizer.invEdgesInTheGraph.containsKey(cell)) {
			//System.out.println("invEdgesInTheGraph");
			this.rightPanel.selectedModelEdges.add(this.visualizer.invEdgesInTheGraph.get(cell));
			this.rightPanel.addLogExtractionPerspective(this.visualizer.invEdgesInTheGraph.get(cell));
			
			System.out.println("this.rightPanel.selectedModelEdges = "+this.rightPanel.selectedModelEdges);
		}
		else if (this.visualizer.invNodesInTheGraph.containsKey(cell)) {
			this.rightPanel.selectedModelActivities.add(this.visualizer.invNodesInTheGraph.get(cell));
			this.rightPanel.addActivityFilterPanel();
			this.rightPanel.addSubgraphVisualizer(this.visualizer.invNodesInTheGraph.get(cell));
			
			System.out.println("this.rightPanel.selectedModelActivities = "+this.rightPanel.selectedModelActivities);
		}
		else {
			this.rightPanel.selectedModelActivities = new HashSet<XOCActivity>();
			this.rightPanel.selectedModelEdges = new HashSet<ModelEdge>();
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
}

class RightPanelStarStar extends JPanel {
	PluginContext context;
	StarStarModel model;
	
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	
	RightPanelSliderStarStar sliders;
	RightPanelLogExtractionPerspective logExtractionPerspective;
	
	RightPanelActivityFilterPanel rightPanelActivityFilterPanel;
	
	SubgraphVisualizer subgraphVisualizer;
	
	Set<ModelEdge> selectedModelEdges;
	Set<XOCActivity> selectedModelActivities;
	
	public RightPanelStarStar(PluginContext context, StarStarModel model) {
		this.context = context;
		this.model = model;
		
		this.selectedModelEdges = new HashSet<ModelEdge>();
		this.selectedModelActivities = new HashSet<XOCActivity>();
		
		RelativeLayout rl = new RelativeLayout(RelativeLayout.Y_AXIS);
		rl.setFill( true );
		this.setLayout(rl);
	}
	
	public void setAware(LeftPanelStarStar leftPanel) {
		this.leftPanel = leftPanel;
		this.visualizer = this.leftPanel.visualizer;
		
		this.sliders = new RightPanelSliderStarStar(context, model, leftPanel, visualizer);
		
		this.add(this.sliders, new Float(33));
	}
	
	public void destroyLogExtractionPerspective() {
		if (this.logExtractionPerspective != null) {
			this.remove(this.logExtractionPerspective);
			this.logExtractionPerspective = null;
			this.updateUI();
		}
	}
	
	public void addLogExtractionPerspective(ModelEdge modelEdge) {
		this.destroyActivityFilterPanel();
		this.destroyLogExtractionPerspective();
		this.destroySubgraphVisualizer();
		
		this.logExtractionPerspective = new RightPanelLogExtractionPerspective(this.context, this.model, this.leftPanel, this.visualizer, this, modelEdge);
		this.add(this.logExtractionPerspective, new Float(33));
		this.updateUI();
	}
	
	public void destroyActivityFilterPanel() {
		if (this.rightPanelActivityFilterPanel != null) {
			this.remove(this.rightPanelActivityFilterPanel);
			this.rightPanelActivityFilterPanel = null;
			this.updateUI();
		}
	}
	
	public void addActivityFilterPanel() {
		this.destroyActivityFilterPanel();
		this.destroyLogExtractionPerspective();
		this.destroySubgraphVisualizer();
		
		this.rightPanelActivityFilterPanel = new RightPanelActivityFilterPanel(context, model, leftPanel, visualizer, this);
		this.add(this.rightPanelActivityFilterPanel, new Float(15));
		this.updateUI();
	}
	
	public void destroySubgraphVisualizer() {
		if (this.subgraphVisualizer != null) {
			this.remove(this.subgraphVisualizer);
			this.subgraphVisualizer = null;
			this.updateUI();
		}
	}
	
	public void addSubgraphVisualizer(XOCActivity clickedActivity) {
		this.destroySubgraphVisualizer();
		this.subgraphVisualizer = new SubgraphVisualizer(context, model, leftPanel, visualizer, this, clickedActivity);
		this.add(this.subgraphVisualizer, new Float(70));
		this.updateUI();
	}
}

class RightPanelActivityFilterPanel extends JPanel {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	
	JButton filterOnSelectedActivitiesButton;
	FilterSelectedActivitiesActionListener filterSelectedActivitiesActionListener;
	
	public RightPanelActivityFilterPanel(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.filterOnSelectedActivitiesButton = new JButton("Filter on all selected activities");
		this.filterSelectedActivitiesActionListener = new FilterSelectedActivitiesActionListener(context, model, leftPanel, visualizer, rightPanel);
		this.filterOnSelectedActivitiesButton.addActionListener(this.filterSelectedActivitiesActionListener);
		
		this.add(this.filterOnSelectedActivitiesButton);
	}
}

class FilterSelectedActivitiesActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	
	public FilterSelectedActivitiesActionListener(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		Set<XOCActivity> wantedActivities = this.rightPanel.selectedModelActivities;
		List<String> wantedActivitiesNames = new ArrayList<String>();
		for (XOCActivity activity : wantedActivities) {
			wantedActivitiesNames.add(activity.getActivityLabel());
		}
		this.model.filterXocOnSpecifiedSetOfActivity(wantedActivitiesNames);
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class RightPanelLogExtractionPerspective extends JPanel {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	
	NiceDoubleSlider weightSlider;
	WeightSliderChangeLog weightSliderChange;
	NiceIntegerSlider logWindow;
	LogWindowChangeLog logWindowChange;
	
	JLabel subwindowTitle;
	JLabel edgePerspective;
	
	JButton extractLog;
	ExtractLogActionListener extractLogActionListener;
	JButton extractPetriInTab;
	ExtractPetriInTabActionListener extractPetriInTabActionListener;
	
	JButton filterModelOnEdgeButton;
	FilterModelOnEdgeActionListener filterModelOnEdgeActionListener;
	
	JButton filterModelOnAllSelectedEdgesButton;
	FilterModelOnAllEdgesActionListener filterModelOnAllEdgesActionListener;
	
	public double getWeightSliderValue() {
		return this.weightSlider.getValue();
	}
	
	public int getLogWindowValue() {
		return this.logWindow.getValue();
	}
	
	public RightPanelLogExtractionPerspective(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.subwindowTitle = new JLabel();
		this.subwindowTitle.setText("Log extraction configuration");
		
		this.edgePerspective = new JLabel();
		this.edgePerspective.setText("Edge perspective: "+this.modelEdge.getRelationClass().getClassLabel());
		
		this.logWindow = SlickerFactory.instance().createNiceIntegerSlider("Log window", ModelConstants.minLogWindow, ModelConstants.maxLogWindow, ModelConstants.defaultLogWindow, Orientation.HORIZONTAL);
		this.logWindowChange = new LogWindowChangeLog(context, model, leftPanel, visualizer, rightPanel, modelEdge, logWindow);
		this.logWindow.addChangeListener(this.logWindowChange);
		
		this.weightSlider = SlickerFactory.instance().createNiceDoubleSlider("Connection weight threshold", ModelConstants.minW, ModelConstants.maxW, ModelConstants.defaultMinConnectionW, Orientation.HORIZONTAL);
		this.weightSliderChange = new WeightSliderChangeLog(context, model, leftPanel, visualizer, rightPanel, modelEdge, weightSlider);
		this.weightSlider.addChangeListener(this.weightSliderChange);
		
		this.extractLog = new JButton("Extract Log in ProM list");
		this.extractLogActionListener = new ExtractLogActionListener(this.context, this.model, this.leftPanel, this.visualizer, this.rightPanel, this.modelEdge, this);
		this.extractLog.addActionListener(this.extractLogActionListener);
		this.extractPetriInTab = new JButton("Extract Petrinet in ProM list");
		this.extractPetriInTabActionListener = new ExtractPetriInTabActionListener(this.context, this.model, this.leftPanel, this.visualizer, this.rightPanel, this.modelEdge, this);
		this.extractPetriInTab.addActionListener(this.extractPetriInTabActionListener);
		
		this.filterModelOnEdgeButton = new JButton("Filter model on last selected edge");
		this.filterModelOnEdgeActionListener = new FilterModelOnEdgeActionListener(this.context, this.model, this.leftPanel, this.visualizer, this.rightPanel, this.modelEdge, this);
		this.filterModelOnEdgeButton.addActionListener(this.filterModelOnEdgeActionListener);
		
		this.filterModelOnAllSelectedEdgesButton = new JButton("Filter model on all selected edges");
		this.filterModelOnAllEdgesActionListener = new FilterModelOnAllEdgesActionListener(context, model, leftPanel, visualizer, rightPanel);
		this.filterModelOnAllSelectedEdgesButton.addActionListener(this.filterModelOnAllEdgesActionListener);
		
		this.add(this.subwindowTitle);
		this.add(this.edgePerspective);
		this.add(this.logWindow);
		this.add(this.weightSlider);
		this.add(this.extractLog);
		this.add(this.extractPetriInTab);
		this.add(this.filterModelOnEdgeButton);
		this.add(this.filterModelOnAllSelectedEdgesButton);
	}
}

class FilterModelOnAllEdgesActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	
	public FilterModelOnAllEdgesActionListener(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		Set<String> edgesInputActivities = new HashSet<String>();
		Set<String> edgesOutputActivities = new HashSet<String>();
		
		for (ModelEdge edge : this.rightPanel.selectedModelEdges) {
			edgesInputActivities.add(edge.getSourceActivity().getActivityLabel());
			edgesOutputActivities.add(edge.getTargetActivity().getActivityLabel());
		}
		
		if (edgesInputActivities.size() == 1 || edgesOutputActivities.size() == 1) {
			this.model.filterXocOnEdges(this.rightPanel.selectedModelEdges, "XOR");
		}
		else {
			this.model.filterXocOnEdges(this.rightPanel.selectedModelEdges, "AND");
		}
		
		this.rightPanel.destroyLogExtractionPerspective();
		this.rightPanel.destroyActivityFilterPanel();
		this.rightPanel.destroySubgraphVisualizer();

		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class FilterModelOnEdgeActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	RightPanelLogExtractionPerspective rightPanelLogExtraction;
	
	public FilterModelOnEdgeActionListener(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge, RightPanelLogExtractionPerspective rightPanelLogExtraction) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		this.rightPanelLogExtraction = rightPanelLogExtraction;
	}

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		String edgeClass = modelEdge.getRelationClass().getClassLabel();
		String edgeSourceActivity = modelEdge.getSourceActivity().getActivityLabel();
		this.model.filterXocOnActivity(edgeSourceActivity, edgeClass);
		this.rightPanel.destroyLogExtractionPerspective();
		this.rightPanel.destroyActivityFilterPanel();
		this.rightPanel.destroySubgraphVisualizer();
		
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class ExtractLogActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	RightPanelLogExtractionPerspective rightPanelLogExtraction;
	
	public ExtractLogActionListener(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge, RightPanelLogExtractionPerspective rightPanelLogExtraction) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		this.rightPanelLogExtraction = rightPanelLogExtraction;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		System.out.println("clicked extract log");
		this.rightPanel.destroyLogExtractionPerspective();
		this.rightPanel.destroyActivityFilterPanel();
		this.rightPanel.destroySubgraphVisualizer();

		LogExtractionFromModel logExtractionFromModel = new LogExtractionFromModel(this.modelEdge.getRelationClass().getClassLabel(), this.model.getIntersectedObjects(), this.model.getXocObjectClasses());
		logExtractionFromModel.applyIterationOnTraces(rightPanelLogExtraction.getWeightSliderValue(), rightPanelLogExtraction.getLogWindowValue());
		XLog log = logExtractionFromModel.getEventualLog();
		
		GetLogFromLog.getLogFromLog(this.context, log);
	}
}

class ExtractPetriInTabActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	RightPanelLogExtractionPerspective rightPanelLogExtraction;
	
	public ExtractPetriInTabActionListener(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge, RightPanelLogExtractionPerspective rightPanelLogExtraction) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		this.rightPanelLogExtraction = rightPanelLogExtraction;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		System.out.println("clicked extract Petri net");
		this.rightPanel.destroyLogExtractionPerspective();
		this.rightPanel.destroyActivityFilterPanel();
		this.rightPanel.destroySubgraphVisualizer();

		LogExtractionFromModel logExtractionFromModel = new LogExtractionFromModel(this.modelEdge.getRelationClass().getClassLabel(), this.model.getIntersectedObjects(), this.model.getXocObjectClasses());
		logExtractionFromModel.applyIterationOnTraces(rightPanelLogExtraction.getWeightSliderValue(), rightPanelLogExtraction.getLogWindowValue());
		XLog log = logExtractionFromModel.getEventualLog();
		System.out.println(log);
		System.out.println(log.size());
		
		for (XTrace tr : log) {
			System.out.println("trace ");
			for (XEvent eve : tr) {
				System.out.println(XExtendedEvent.wrap(eve).getName());
			}
			System.out.println("\n\n");
		}
		
		Object[] petri = GetNetFromLogUsingIM.mineProcessUsingIM(this.context, log);
		Petrinet net = (Petrinet) petri[0];
		Marking marking = (Marking) petri[1];
		
		PetriNetVisualizerTab visualizerTab = new PetriNetVisualizerTab(this.context, this.model, net, marking);
		
		String edgeLabel = "Petrinet (p: "+this.modelEdge.getRelationClass().getClassLabel()+" we:"+String.format("%.3f", rightPanelLogExtraction.getWeightSliderValue())+" wi:"+rightPanelLogExtraction.getLogWindowValue();
		
		this.leftPanel.addJPanel(edgeLabel,visualizerTab);
		this.leftPanel.updateUI();
	}
	
}

class WeightSliderChangeLog implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	NiceDoubleSlider weightSlider;
	
	public WeightSliderChangeLog(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge, NiceDoubleSlider weightSlider) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		this.weightSlider = weightSlider;
	}

	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		
	}
}

class LogWindowChangeLog implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	RightPanelStarStar rightPanel;
	ModelEdge modelEdge;
	NiceIntegerSlider logWindow;
	
	public LogWindowChangeLog(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer, RightPanelStarStar rightPanel, ModelEdge modelEdge, NiceIntegerSlider logWindow) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.modelEdge = modelEdge;
		this.logWindow = logWindow;
	}

	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		
	}
}

class RightPanelSliderStarStar extends JPanel {
	PluginContext context;
	StarStarModel model;
	
	LeftPanelStarStar leftPanel;
	ModelVisualizerStarStar visualizer;
	
	JLabel title;
	NiceDoubleSlider weightSlider;
	WeightSliderChange weightSliderChange;
	NiceIntegerSlider minCountSelector;
	MinCountSliderChange minCountSliderChange;
	NiceIntegerSlider parallelEdgeSpacing;
	ParallelEdgeSliderChange parallelEdgeSliderChange;
	NiceIntegerSlider interrankCellSpacing;
	InterrankCellSpacingChange interrankCellSpacingChange;
	NiceIntegerSlider interHierarchySpacing;
	InterHierarchySpacingListener interHierarchySpacingListener;
	NiceIntegerSlider intraCellSpacing;
	IntraCellSpacingListener intraCellSpacingListener;
	JCheckBox checkBoxActivityCountInLabel;
	CheckboxChangeListener activityCountInLabelListener;
	NiceIntegerSlider minActivityCountSelector;
	MinActivityCountSliderChange minActivityCountSliderChangeListener;
	
	JLabel metric;
	JComboBox<String> metricSelection;
	
	MetricSelectionChange metricSelectionChange;
	
	JButton resetModelButton;
	ResetModelActionListener resetModelActionListener;
	
	JButton setCheckpointModelButton;
	CheckpointModelActionListener checkPointModelActionListener;
	
	public RightPanelSliderStarStar(PluginContext context, StarStarModel model, LeftPanelStarStar leftPanel, ModelVisualizerStarStar visualizer) {
		this.context = context;
		this.model = model;
		this.leftPanel = leftPanel;
		this.visualizer = visualizer;
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.resetModelButton = new JButton("Reset model to checkpoint");
		this.resetModelActionListener = new ResetModelActionListener(context, model, visualizer, this);
		this.resetModelButton.addActionListener(this.resetModelActionListener);
		this.add(this.resetModelButton);
		
		this.setCheckpointModelButton = new JButton("Save current model as checkpoint");
		this.checkPointModelActionListener = new CheckpointModelActionListener(context, model, visualizer, this);
		this.setCheckpointModelButton.addActionListener(this.checkPointModelActionListener);
		this.add(this.setCheckpointModelButton);
		
		this.title = new JLabel("Model parameters:");
		this.add(this.title);
		
		this.minActivityCountSelector = SlickerFactory.instance().createNiceIntegerSlider("Min activity count", ModelConstants.minMinActivityCount, this.visualizer.maxActivityCount, ModelConstants.defaultMinActivityCount, Orientation.HORIZONTAL);
		this.minActivityCountSliderChangeListener = new MinActivityCountSliderChange(context, model, visualizer, this, this.minActivityCountSelector);
		this.minActivityCountSelector.addChangeListener(this.minActivityCountSliderChangeListener);
		this.add(this.minActivityCountSelector);
				
		this.minCountSelector = SlickerFactory.instance().createNiceIntegerSlider("Min path count", ModelConstants.minMinCount, this.visualizer.maxCountPerEdge, ModelConstants.defaultMinCount, Orientation.HORIZONTAL);
		this.minCountSliderChange = new MinCountSliderChange(context, model, visualizer, this, this.minCountSelector);
		this.minCountSelector.addChangeListener(this.minCountSliderChange);
		this.add(this.minCountSelector);
		
		this.weightSlider = SlickerFactory.instance().createNiceDoubleSlider("Weight threshold", ModelConstants.minW, ModelConstants.maxW, ModelConstants.defaultW, Orientation.HORIZONTAL);
		this.weightSliderChange = new WeightSliderChange(context, model, visualizer, this, weightSlider);
		this.weightSlider.addChangeListener(this.weightSliderChange);
		this.add(this.weightSlider);
		
		this.parallelEdgeSpacing = SlickerFactory.instance().createNiceIntegerSlider("Parallel edge spacing", ModelConstants.minParallelEdgeSpacing, ModelConstants.maxParallelEdgeSpacing, ModelConstants.defaultParallelEdgeSpacing, Orientation.HORIZONTAL);
		this.parallelEdgeSliderChange = new ParallelEdgeSliderChange(context, model, visualizer, this, parallelEdgeSpacing);
		this.parallelEdgeSpacing.addChangeListener(this.parallelEdgeSliderChange);
		this.add(this.parallelEdgeSpacing);
		
		this.interrankCellSpacing = SlickerFactory.instance().createNiceIntegerSlider("Inter rank cell spacing", ModelConstants.minParallelEdgeSpacing, ModelConstants.maxParallelEdgeSpacing, ModelConstants.defaultParallelEdgeSpacing, Orientation.HORIZONTAL);
		this.interrankCellSpacingChange = new InterrankCellSpacingChange(context, model, visualizer, this, interrankCellSpacing);
		this.interrankCellSpacing.addChangeListener(this.interrankCellSpacingChange);
		this.add(this.interrankCellSpacing);
		
		this.interHierarchySpacing = SlickerFactory.instance().createNiceIntegerSlider("Inter hierarchy spacing", ModelConstants.minInterHierarchySpacing, ModelConstants.maxInterHierarchySpacing, ModelConstants.defaultInterHierarchySpacing, Orientation.HORIZONTAL);
		this.interHierarchySpacingListener = new InterHierarchySpacingListener(context, model, visualizer, this, interHierarchySpacing);
		this.interHierarchySpacing.addChangeListener(this.interHierarchySpacingListener);
		//this.add(this.interHierarchySpacing);
		
		this.intraCellSpacing = SlickerFactory.instance().createNiceIntegerSlider("Intra cell spacing", ModelConstants.minIntraCellSpacing, ModelConstants.maxIntraCellSpacing, ModelConstants.defaultIntraCellSpacing, Orientation.HORIZONTAL);
		this.intraCellSpacingListener = new IntraCellSpacingListener(context, model, visualizer, this, intraCellSpacing);
		this.intraCellSpacing.addChangeListener(this.intraCellSpacingListener);
		this.add(this.intraCellSpacing);
		
		this.checkBoxActivityCountInLabel = new JCheckBox("Enable Activity count in label");
		this.checkBoxActivityCountInLabel.setSelected(this.visualizer.showCountOnActivity);
		this.activityCountInLabelListener = new CheckboxChangeListener(context, model, visualizer, this, this.checkBoxActivityCountInLabel);
		this.checkBoxActivityCountInLabel.addItemListener(this.activityCountInLabelListener);
		this.add(this.checkBoxActivityCountInLabel);
		
		this.metric = new JLabel("metric: ");
		this.add(this.metric);
		
		String[] metrics = { "count", "weight", "performance", "type" };
		
		this.metricSelection = new JComboBox() {
            /** 
             * @inherited <p>
             */
            @Override
            public Dimension getMaximumSize() {
                Dimension max = super.getMaximumSize();
                max.height = getPreferredSize().height;
                return max;
            }
        };
        this.metricSelection.setPreferredSize(new Dimension(150, 30));
        for (String metric : metrics) {
        	this.metricSelection.addItem(metric);
        }
		this.metricSelection.setSelectedItem(ModelConstants.defaultPerspective);
		this.add(this.metricSelection);
		
		this.metricSelectionChange = new MetricSelectionChange(context, model, visualizer, this, this.metricSelection);
		this.metricSelection.addActionListener(this.metricSelectionChange);
	}
}

class CheckpointModelActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	
	public CheckpointModelActionListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		this.model.setCheckpointModel();
	}
}
class ResetModelActionListener implements ActionListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	
	public ResetModelActionListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		this.model.restoreUnfiltered();
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
	
}

class MinActivityCountSliderChange implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider minActivityCountSelector;
	
	public MinActivityCountSliderChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider minActivityCountSelector) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.minActivityCountSelector = minActivityCountSelector;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setMinActivityCount(this.minActivityCountSelector.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class MinCountSliderChange implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider minCountSelector;
	
	public MinCountSliderChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider minCountSelector) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.minCountSelector = minCountSelector;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setMinCount(this.minCountSelector.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class CheckboxChangeListener implements ItemListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	JCheckBox checkBoxActivityCountInLabel;
	
	public CheckboxChangeListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, JCheckBox checkBoxActivityCountInLabel) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.checkBoxActivityCountInLabel = checkBoxActivityCountInLabel;
	}
	
	public void itemStateChanged(ItemEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setShowCountOnActivities(this.checkBoxActivityCountInLabel.isSelected());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class IntraCellSpacingListener implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider intraCellSpacing;
	
	public IntraCellSpacingListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider intraCellSpacing) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.intraCellSpacing = intraCellSpacing;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setIntraCellSpacing(this.intraCellSpacing.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class InterHierarchySpacingListener implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider interHierarchySpacing;
	
	public InterHierarchySpacingListener(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider interHierarchySpacing) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.interHierarchySpacing = interHierarchySpacing;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub	
		this.visualizer.setInterHierarchySpacing(this.interHierarchySpacing.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class MetricSelectionChange implements ActionListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	JComboBox<String> metricSelection;
	
	public MetricSelectionChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, JComboBox<String> metricSelection) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.metricSelection = metricSelection;
	}

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setPerspective((String)this.metricSelection.getSelectedItem()); 
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class ParallelEdgeSliderChange implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider parallelEdgeSpacing;
	
	public ParallelEdgeSliderChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider parallelEdgeSpacing) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.parallelEdgeSpacing = parallelEdgeSpacing;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setParallelEdgeSpacing(this.parallelEdgeSpacing.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class InterrankCellSpacingChange implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceIntegerSlider interrankCellSpacing;
	
	public InterrankCellSpacingChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceIntegerSlider interrankCellSpacing) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.interrankCellSpacing = interrankCellSpacing;
	}

	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setInterRankCellSpacing(this.interrankCellSpacing.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
}

class WeightSliderChange implements ChangeListener {
	PluginContext context;
	StarStarModel model;
	ModelVisualizerStarStar visualizer;
	RightPanelSliderStarStar rightPanel;
	NiceDoubleSlider weightSlider;
	
	public WeightSliderChange(PluginContext context, StarStarModel model, ModelVisualizerStarStar visualizer, RightPanelSliderStarStar rightPanel, NiceDoubleSlider weightSlider) {
		this.context = context;
		this.model = model;
		this.visualizer = visualizer;
		this.rightPanel = rightPanel;
		this.weightSlider = weightSlider;
	}
	
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
		this.visualizer.setW(this.weightSlider.getValue());
		this.visualizer.doRepresentationWork();
		this.visualizer.addGraphToView();
	}
	
}

class PetriNetVisualizerTab extends JPanel {
	PluginContext context;
	StarStarModel model;
	Petrinet net;
	Marking marking;
	
	ScalableComponent scalable;
	ProMJGraph graph;
	JComponent graphComponent;
		
	public PetriNetVisualizerTab(PluginContext context, StarStarModel model, Petrinet net, Marking marking) {
		this.context = context;
		this.model = model;
		this.net = net;
		this.marking = marking;
		
		RelativeLayout rl = new RelativeLayout(RelativeLayout.Y_AXIS);
		rl.setFill( true );
		this.setLayout(rl);
		
		visualize();
	}
	
	public void constructVisualization(ViewSpecificAttributeMap map, boolean isShowMoveLogModel, boolean isShowMoveModel) {
		graph.getModel().beginUpdate();

		//doUpdateInternal(map, isShowMoveLogModel, isShowMoveModel);

		graph.getModel().endUpdate();
		graph.refresh();
		graph.revalidate();
		graph.repaint();
	}
	
	JComponent graphVisualize() {
		scalable = GraphBuilder.buildJGraph(net);
		graph = (ProMJGraph) scalable;
		
		constructVisualization(graph.getViewSpecificAttributes(), true, true);
		
		return graph.getComponent();
	}
	
	public void visualize() {
		this.graphComponent = graphVisualize();
		
		this.add(graphComponent, new Float(100));
	}
}